"""
This module contains just a lookup table for the local Clifford group.
"""

C_L = ['H'
    , 'S'
    , 'ZZ'
    , 'SH'
    , 'HS'
    , 'Z'
    , 'SHS'
    , 'HZ'
    , 'ZS'
    , 'SXH'
    , 'HSZ'
    , 'HXSH'
    , 'HSH'
    , 'HX'
    , 'X'
    , 'ZSH'
    , 'SX'
    , 'HXS'
    , 'XS'
    , 'SZHS'
    , 'ZXH'
    , 'XZ'
    , 'XSH'
    , 'HSX']
